from typing import Callable

from google.adk.agents.callback_context import CallbackContext
from google.adk.events.event import Event
from google.adk.flows.llm_flows.contents import _get_contents
from google.adk.models.llm_request import LlmRequest
from google.adk.models.llm_response import LlmResponse
from google.adk.plugins.base_plugin import BasePlugin
from google.genai import types


class ManageContextPlugin(BasePlugin):
    def __init__(
        self,
        clear_after_transfer: bool | list[str] = False,
        filter_events: Callable[[list[Event], CallbackContext], list[Event]]
        | None = None,
    ):
        super().__init__(name="manage_context")
        self.clear_after_transfer = clear_after_transfer
        self.filter_events = filter_events

    async def before_model_callback(
        self, *, callback_context: CallbackContext, llm_request: LlmRequest
    ) -> LlmResponse | None:
        events = callback_context._invocation_context.session.events
        if not events:
            return None

        if self.clear_after_transfer:
            if (
                not isinstance(self.clear_after_transfer, list)
                or callback_context.agent_name in self.clear_after_transfer
            ):
                start_index = 0
                for idx, event in reversed(list(enumerate(events))):
                    if (
                        event.author != callback_context.agent_name
                        and event.author != "user"
                    ):
                        start_index = min(idx + 1, len(events) - 1)
                        break
                events = events[start_index:]

        if self.filter_events:
            events = self.filter_events(events, callback_context)

        llm_request.contents = _get_contents(
            callback_context._invocation_context.branch,
            events,
            callback_context.agent_name,
        )

        if not llm_request.contents or llm_request.contents[0].role != "user":
            llm_request.contents = [
                types.Content(
                    role="user",
                    parts=[
                        types.Part(
                            text="Handle the requests as specified in the System Instruction."
                        )
                    ],
                )
            ] + llm_request.contents



from .base import queued_job
from .config import (
    create_redis_async_client,
    create_redis_client,
    get_job_queue_broker,
    reset_broker,
)
from .progress import notify_job_progress, queue_job_stream_service

__all__ = [
    "queued_job",
    "get_job_queue_broker",
    "create_redis_async_client",
    "create_redis_client",
    "reset_broker",
    "notify_job_progress",
    "queue_job_stream_service",
]



from pydantic import BaseModel, ValidationError
from pydantic.alias_generators import to_snake

from sparkden.models.assistant import (
    ToolResponse,
    ToolResponseStatus,
)


def validate_tool_param(
    tool_name: str, param_value: dict, param_type: type[BaseModel]
) -> ToolResponse | None:
    try:
        param_type.model_validate(param_value)
    except ValidationError as e:
        error_str = f"""Invoking `{tool_name}()` failed as the input parameters validation failed:
{str(e)}
You could retry calling this tool, but it is IMPORTANT for you to follow the input parameters schema."""
        return ToolResponse(
            status=ToolResponseStatus.ERROR,
            error=error_str,
        )
    return None


def assistant_context_collection_id(assistant_id: str) -> str:
    return f"{to_snake(assistant_id).replace('_', '-')}-context"



"""
Configures the job queue broker and Redis clients
"""

import os

import dramatiq
import redis
import redis.asyncio as aioredis
from dramatiq.brokers.redis import RedisBroker
from dramatiq.middleware import AsyncIO
from dramatiq.results import Results
from dramatiq.results.backends.redis import RedisBackend


def get_redis_url() -> str:
    """Get Redis URL."""
    return os.getenv("REDIS_URL", "redis://localhost:6379")


def create_redis_client() -> redis.Redis:
    """Create synchronous Redis client (for Dramatiq)."""
    return redis.from_url(get_redis_url())


def create_redis_async_client() -> aioredis.Redis:
    """Create async Redis client (for progress notifications and SSE)."""
    return aioredis.from_url(get_redis_url())


def setup_job_queue_broker() -> RedisBroker:
    """Setup job queue broker."""
    redis_url = get_redis_url()

    broker = RedisBroker(url=redis_url)

    # Add async middleware for async job execution
    broker.add_middleware(AsyncIO())

    # Add result backend
    result_backend = RedisBackend(url=redis_url)
    broker.add_middleware(Results(backend=result_backend))

    dramatiq.set_broker(broker)
    return broker


# Global broker instance (lazy init)
_broker = None


def get_job_queue_broker() -> RedisBroker:
    """Get job queue broker (singleton)."""
    global _broker
    if _broker is None:
        _broker = setup_job_queue_broker()
    return _broker


def reset_broker():
    """Reset broker (for testing)."""
    global _broker
    _broker = None
